<?php
namespace Academy\API;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

use Academy\Traits\LessonSchema;

class Lessons extends \WP_REST_Controller {

	use LessonSchema;

	public static function init() {
		$self            = new self();
		$self->namespace = ACADEMY_PLUGIN_SLUG . '/v1';
		$self->rest_base = 'lessons';
		add_action( 'rest_api_init', array( $self, 'register_routes' ) );
	}

	/**
	 * Register the routes for the objects of the controller.
	 */
	public function register_routes() {
		register_rest_route(
			$this->namespace,
			'/' . $this->rest_base,
			array(
				array(
					'methods'             => \WP_REST_Server::READABLE,
					'callback'            => array( $this, 'get_items' ),
					'permission_callback' => array( $this, 'permissions_check' ),
					'args'                => $this->get_collection_params(),
				),
				array(
					'methods'             => \WP_REST_Server::CREATABLE,
					'callback'            => array( $this, 'create_item' ),
					'permission_callback' => array( $this, 'permissions_check' ),
					'args'                => $this->get_item_schema(),
				),
				'schema' => array( $this, 'get_public_item_schema' ),
			)
		);

		$schema        = $this->get_item_schema();
		$get_item_args = array(
			'context' => $this->get_context_param( array( 'default' => 'view' ) ),
		);
		if ( isset( $schema['properties']['password'] ) ) {
			$get_item_args['password'] = array(
				'description' => esc_html__( 'The password for the post if it is password protected.', 'academy' ),
				'type'        => 'string',
			);
		}

		register_rest_route(
			$this->namespace,
			'/' . $this->rest_base . '/(?P<id>[\d]+)',
			array(
				'args'   => array(
					'id' => array(
						'description' => esc_html__( 'Unique identifier for the object.', 'academy' ),
						'type'        => 'integer',
					),
				),
				array(
					'methods'             => \WP_REST_Server::READABLE,
					'callback'            => array( $this, 'get_item' ),
					'permission_callback' => array( $this, 'permissions_check' ),
					'args'                => $get_item_args,
				),
				array(
					'methods'             => \WP_REST_Server::EDITABLE,
					'callback'            => array( $this, 'update_item' ),
					'permission_callback' => array( $this, 'permissions_check' ),
					'args'                => $this->get_item_schema(),
				),
				array(
					'methods'             => \WP_REST_Server::DELETABLE,
					'callback'            => array( $this, 'delete_item' ),
					'permission_callback' => array( $this, 'permissions_check' ),
					'args'                => array(
						'force' => array(
							'type'        => 'boolean',
							'default'     => false,
							'description' => esc_html__( 'Whether to bypass Trash and force deletion.', 'academy' ),
						),
					),
				),
				'schema' => array( $this, 'get_public_item_schema' ),
			)
		);
	}


	public function permissions_check( $request ) {
		if ( ! current_user_can( 'manage_academy_instructor' ) ) {
			return new \WP_Error(
				'rest_forbidden_context',
				esc_html__( 'Sorry, you are not allowed to edit posts in this post type.', 'academy' ),
				array( 'status' => rest_authorization_required_code() )
			);
		}
		return true;
	}


	/**
	 * Retrieves a collection of posts.
	 *
	 * @since 4.7.0
	 *
	 * @param WP_REST_Request $request Full details about the request.
	 * @return WP_REST_Response|WP_Error Response object on success, or \WP_Error object on failure.
	 */
	public function get_items( $request ) {
		global $wpdb;
		$results = [];
		if ( current_user_can( 'manage_options' ) ) {
			$lessons = $wpdb->get_results( "SELECT * FROM {$wpdb->prefix}academy_lessons ORDER BY lesson_date DESC", OBJECT );
		} elseif ( current_user_can( 'manage_academy_instructor' ) ) {
			$user_id = get_current_user_id();
			$lessons = $wpdb->get_results( $wpdb->prepare( "SELECT * FROM {$wpdb->prefix}academy_lessons WHERE lesson_author=%d ORDER BY lesson_date DESC", $user_id ), OBJECT );
		}

		if ( is_array( $lessons ) ) {
			foreach ( $lessons as $lesson ) {
				$lesson->lesson_content = stripslashes( $lesson->lesson_content );
				$lesson->author_username = get_the_author_meta( 'user_login', $lesson->lesson_author );
				$lesson->meta = \Academy\Helper::get_lesson_meta_data( $lesson->ID );
				$results[]               = $lesson;
			}
		}
		return new \WP_REST_Response(
			$results,
			200
		);
	}

	public function get_item( $request ) {
		$ID = (int) $request->get_param( 'id' );
		$lesson = \Academy\Helper::get_lesson( $ID );
		return new \WP_REST_Response(
			$lesson,
			200
		);
	}



	/**
	 * Creates a single post.
	 *
	 * @since 4.7.0
	 *
	 * @param WP_REST_Request $request Full details about the request.
	 * @return WP_REST_Response|WP_Error Response object on success, or \WP_Error object on failure.
	 */
	public function create_item( $request ) {
		$params    = wp_slash( (array) $request->get_params() );
		$lesson_id = $this->lesson_insert( $params );
		if ( $lesson_id ) {
			$this->lesson_meta_insert( $lesson_id, $params['meta'] );
			$lesson = \Academy\Helper::get_lesson( $lesson_id );
			return new \WP_REST_Response( current( $lesson ), 200 );
		}
		return new \WP_REST_Response( [] );
	}

	public function update_item( $request ) {
		$params    = wp_slash( (array) $request->get_params() );
		$lesson_id = $this->lesson_insert( $params );
		$this->lesson_meta_update( $lesson_id, $params['meta'] );
		$lesson = \Academy\Helper::get_lesson( $lesson_id );
		return new \WP_REST_Response( current( $lesson ), 200 );
	}

	public function delete_item( $request ) {
		$params = $request->get_params();
		try {
			global $wpdb;
			$wpdb->delete(
				$wpdb->prefix . 'academy_lessons',
				array(
					'ID' => $params['id'],
				),
				array(
					'%d',
				)
			);
			$wpdb->delete(
				$wpdb->prefix . 'academy_lessonmeta',
				array(
					'lesson_id' => $params['id'],
				),
				array(
					'%d',
				)
			);
			return new \WP_REST_Response( $params['id'], 200 );
		} catch ( \Throwable $th ) {
			return new \WP_REST_Response( $th->getMessage(), 200 );
		}//end try
	}

	public function lesson_insert( $postarr ) {
		if ( ! is_array( $postarr ) ) {
			return null;
		}

		global $wpdb;
		$user_id  = get_current_user_id();
		$defaults = array(
			'lesson_author'       => $user_id,
			'lesson_date'         => '',
			'lesson_date_gmt'     => '',
			'lesson_title'        => '',
			'lesson_name'        => '',
			'lesson_content'      => '',
			'lesson_excerpt'      => '',
			'lesson_status'       => 'draft',
			'comment_status'      => 'close',
			'comment_count'       => 0,
			'lesson_password'     => '',
			'lesson_modified'     => current_time( 'mysql' ),
			'lesson_modified_gmt' => current_time( 'mysql' ),
		);

		$lookFor = array_combine( array_keys( $this->get_item_schema() ), array_keys( $this->get_item_schema() ) );
		$postarr = array_intersect_key( $postarr, $lookFor );

		$lessonarr = wp_parse_args( $postarr, $defaults );

		// Are we updating or creating?
		$lesson_ID = 0;
		$update    = false;

		if ( ! empty( $postarr['ID'] ) ) {
			$lesson_ID = $postarr['ID'];
			$update    = true;
			unset( $lessonarr['ID'] );
		} else {
			$lessonarr['lesson_date']     = current_time( 'mysql' );
			$lessonarr['lesson_date_gmt'] = current_time( 'mysql' );
		}

		// post insert will be here
		$table_name = $wpdb->prefix . 'academy_lessons';
		if ( $update ) {
			$wpdb->update(
				$table_name,
				$lessonarr,
				array( 'ID' => $lesson_ID ),
				array(
					'%d',
					'%s',
					'%s',
					'%s',
					'%s',
					'%s',
					'%s',
					'%s',
					'%d',
					'%s',
					'%s',
					'%s',
				),
				array( '%d' )
			);
			return $lesson_ID;
		} else {
			$wpdb->insert(
				$table_name,
				array(
					'lesson_author' => $lessonarr['lesson_author'],
					'lesson_date' => $lessonarr['lesson_date'],
					'lesson_date_gmt' => $lessonarr['lesson_date_gmt'],
					'lesson_title' => $lessonarr['lesson_title'],
					'lesson_name' => $lessonarr['lesson_name'],
					'lesson_content' => $lessonarr['lesson_content'],
					'lesson_excerpt' => $lessonarr['lesson_excerpt'],
					'lesson_status' => $lessonarr['lesson_status'],
					'comment_status' => $lessonarr['comment_status'],
					'comment_count' => $lessonarr['comment_count'],
					'lesson_password' => $lessonarr['lesson_password'],
					'lesson_modified' => $lessonarr['lesson_modified'],
					'lesson_modified_gmt' => $lessonarr['lesson_modified_gmt'],
				),
				array(
					'%d',
					'%s',
					'%s',
					'%s',
					'%s',
					'%s',
					'%s',
					'%s',
					'%s',
					'%d',
					'%s',
					'%s',
					'%s',
				)
			);
			return $wpdb->insert_id;
		}//end if
		return null;
	}

	public function lesson_meta_insert( $lesson_id, $items ) {
		global $wpdb;
		$table_name = $wpdb->prefix . 'academy_lessonmeta';
		if ( is_array( $items ) && count( $items ) > 0 ) {
			foreach ( $items as $key => $value ) {
				if ( is_array( $value ) ) {
					$value = wp_json_encode( $value );
				}
				$wpdb->insert(
					$table_name,
					array(
						'lesson_id'     => $lesson_id,
						'meta_key'      => $key,
						'meta_value'    => $value,
					),
					array(
						'%d',
						'%s',
						'%s',
					)
				);
			}
		}
	}

	public function lesson_meta_update( $lesson_id, $items ) {
		global $wpdb;
		$table_name = $wpdb->prefix . 'academy_lessonmeta';
		if ( is_array( $items ) && count( $items ) > 0 ) {
			foreach ( $items as $key => $value ) {
				if ( is_array( $value ) ) {
					$value = wp_json_encode( $value );
				}
				if ( count(
					$wpdb->get_results(
						$wpdb->prepare( "SELECT lesson_id, meta_key  FROM {$wpdb->prefix}academy_lessonmeta WHERE lesson_id=%d AND meta_key=%s", intval( $lesson_id ), $key )
					)
				) > 0 ) {
					// update will be here
					$wpdb->update(
						$table_name,
						array(
							'meta_key'   => $key,
							'meta_value' => $value,
						),
						array(
							'lesson_id' => $lesson_id,
							'meta_key'  => $key,
						),
						array(
							'%s',
							'%s',
						),
						array( '%d', '%s' )
					);
				}
			}//end foreach
		}//end if
	}
}
